@@ -23,20 +23,24 @@
 /* Initialize the following variables.  */
 void glyphs_init (void);
 
+/* In gnulib/lib/unicodeio.h unicode_to_mb uses a buffer of 25 bytes.
+   In down_arrow, we append one space.  */
+typedef char glyph_buffer_t[26];
+
 /* "→", separates the lhs of a rule from its rhs.  */
-extern const char *arrow;
+extern glyph_buffer_t arrow;
 extern int arrow_width;
 
 /* "•", a point in an item (aka, a dotted rule).  */
-extern const char *dot;
+extern glyph_buffer_t dot;
 extern int dot_width;
 
 /* "↳ ", below an lhs to announce the rhs.  */
-extern const char *down_arrow;
+extern glyph_buffer_t down_arrow;
 extern int down_arrow_width;
 
 /* "ε", an empty rhs.  */
-extern const char *empty;
+extern glyph_buffer_t empty;
 extern int empty_width;
 
 /* " ", separate symbols in the rhs of a derivation.  */
